% Generate test and possible sample points for the next iteration
% by using MCMC (Gibbs sampler) to draw samples from the variance of the
% interest distribution.  Then follow-up with Gaussian blur.
%
% Separatrix Demo, Institute for Disease Modeling, May 2014

function [mcmcTestPoints, mcmcPossibleSamplePoints] = ChooseTestAndSamplePointsForNextIteration(testPoints, Interest_x, InterestDistributionTestPoints, InterestDistributionInferencePoints, possibleSamplePoints, ParameterRanges, params)

mcmcTestPoints = testPoints;
mcmcPossibleSamplePoints = possibleSamplePoints;
MCMCres = params.MCMC_Resolution;
NpossibleSamplePoints = size(possibleSamplePoints,1);

% Note that we are using the variance at both the test points and at the inference points.
% Coordinates:
FullInterestVariancePoints = [testPoints; Interest_x];
% Interest distribution variance:
FullInterestDistribution = [InterestDistributionTestPoints; InterestDistributionInferencePoints];
FullInterestVariance = FullInterestDistribution(:,1)-FullInterestDistribution(:,1).^2;
% Avoid duplicates
[UniqueInterestVariancePoints, UniqueInterestVariancePointsInds] = unique(FullInterestVariancePoints, 'rows');
UniqueInterestVariance(:,1) = FullInterestVariance(UniqueInterestVariancePointsInds);

% Want to infer the variance of the interest distribution at MCMCres points.
% Instead of directly calculating the variance, we interpolate.
% However, this interpolation can go badly due to the fact that probability
% is constrained to [0,1].  To address this constraint, we do the interpolation
% in logit space and then return to linear space.

% Interpolation gets confused at -Inf and Inf logit,
% so map probability to ( smallNumber, 1-smallNumber )
logitInterestVariance = log(UniqueInterestVariance ./ (1-UniqueInterestVariance) );

if params.Num_Dimensions == 1
	InterpPts(:,1) = linspace(0,1,MCMCres);
	InterpPts(:,1) = LHSPointSelection(MCMCres, 1, ParameterRanges);

	% Draw params.Num_Test_Points from the interpolation
	interpval = interp1(UniqueInterestVariancePoints,logitInterestVariance,InterpPts);
	
	InterpolatedInterestVariancePDF = 1./(1+exp(-interpval));
	
	% Interpolation gives NaN sometimes, set these to zero:
	inds = isnan(InterpolatedInterestVariancePDF);
	InterpolatedInterestVariancePDF(inds) = 0;
	
	% Replace points with the randomly selected values:
	for i = 1:params.Num_Test_Points
		mcmcTestPoints(i,1) = InterpPts( roulette(InterpolatedInterestVariancePDF) );
	end
	for i = 1:NpossibleSamplePoints
		mcmcPossibleSamplePoints(i,1) = InterpPts( roulette(InterpolatedInterestVariancePDF) );
	end
	
else
	zinterp = scatteredInterpolant(UniqueInterestVariancePoints, logitInterestVariance);
	for iteration=1:params.MCMC_Num_Iterations
		for dimension = 1:params.Num_Dimensions
			InterpPts(:,1) = LHSPointSelection(MCMCres, 1, ParameterRanges);
			% Interpolate
			for ii = 1:params.Num_Test_Points
				ip = repmat(mcmcTestPoints(ii,:), MCMCres, 1);
				ip(:,dimension) = InterpPts;
				InterpolatedInterestVariancePDF = 1./(1+exp(-zinterp(ip)));

				% Remove interpolated NaNs
				inds = isnan(InterpolatedInterestVariancePDF);
				InterpolatedInterestVariancePDF(inds) = 0;

				mcmcTestPoints(ii,dimension) = InterpPts( roulette(InterpolatedInterestVariancePDF, MCMCres) );	% Replace testpoint with the selected value
			end

			for ii = 1:NpossibleSamplePoints
				sp = repmat(mcmcPossibleSamplePoints(ii,:), MCMCres, 1);
				sp(:,dimension) = InterpPts;
				InterpolatedInterestVariancePDF = 1./(1+exp(-zinterp(sp)));

				% Remove interpolated NaNs
				inds = isnan(InterpolatedInterestVariancePDF);
				InterpolatedInterestVariancePDF(inds) = 0;

				mcmcPossibleSamplePoints(ii,dimension) = InterpPts( roulette(InterpolatedInterestVariancePDF) );
			end
		end
	end
end

if params.Blur_Sigma > 0
	% Apply Gaussian blur
	for dimension = 1:params.Num_Dimensions
		tp_blur = mcmcTestPoints(:,dimension) + params.Blur_Sigma * randn(params.Num_Test_Points,1);
		inds = find(tp_blur < 0 | tp_blur > 1);
		tp_blur(inds) = mcmcTestPoints(inds,dimension);
		mcmcTestPoints(:,dimension) = tp_blur;
		
		psp_blur = mcmcPossibleSamplePoints(:,dimension) + params.Blur_Sigma * randn(NpossibleSamplePoints,1);
		inds = find(psp_blur < ParameterRanges{dimension}.min | psp_blur > ParameterRanges{dimension}.max);
		psp_blur(inds) = mcmcPossibleSamplePoints(inds,dimension);
		mcmcPossibleSamplePoints(:,dimension) = psp_blur;
	end
end

% HELPER FUNCTIONS:
	function idx = roulette(pdf, MCMCres)
		% Roulette wheel selection from pdf
		cdf = cumsum(pdf);
		r = rand * cdf(end);
		idx = find(cdf > r, 1, 'first');
		if isempty(idx)
			idx = MCMCres;
		end
	end

end