% Demonstration of the separatrix algorithm.
% Main.m calls this function to perform the analysis
%
% Separatrix Demo, Institute for Disease Modeling, May 2014

function SeparatrixDemo(params)

%% Clean from previous runs
if ~exist(params.Output_Dir, 'dir')
	mkdir(params.Output_Dir);	% Make the output dir
end

% Clear cached test and sample points:
fn = fullfile(params.Output_Dir, 'mcmcTestPoints.mat');
if exist(fn, 'file')
	delete( fn );
end

fn = fullfile(params.Output_Dir, 'mcmcPossibleSamplePoints.mat');
if exist(fn, 'file')
	delete( fn );
end

%% Initialization

% Model
model_config.RandomSeed = params.Random_Seed;
model = eval( sprintf('%s(model_config)', params.Model) );

ParamNames = model.Parameter_Names;
ParameterRanges = model.Parameter_Ranges;
params.Num_Dimensions = length(ParamNames);	% Store number of dimensions

% Set random number seed
rng(params.Random_Seed);

% Generate iniital sample points, inference points, and initial outcomes:
num_samples = params.Num_Initial_Samples;	
grid_res = params.Inference_Grid_Resolution;
sample_x = LHSPointSelection(num_samples, params.Num_Dimensions, ParameterRanges);
if params.Num_Dimensions == 1
	inference_x = linspace(0, ParameterRanges{1}.max, grid_res)';
else % 2-D
	[ix, iy] = meshgrid( linspace(0, ParameterRanges{1}.max, grid_res), linspace(0, ParameterRanges{2}.max, grid_res) );
	inference_x = [ix(:), iy(:)];
end

% Compute true/false outputs at the initial sample points
sample_y = model.Sample(sample_x);

% True Separatrix
true_separatrix = model.TrueSeparatrix(params.Interest_Level);

%% Perform initial Separatrix inference
[Alpha, Beta] = SeparatrixInference(sample_x, sample_y, inference_x, params);

% Initial plotting
h = figure(3); clf; set(h, 'color', 'w', 'name', 'Separatrix Analysis');
if params.Num_Dimensions == 1
	plotInference1D(inference_x, sample_x, [], sample_y, [], Alpha, Beta, ParameterRanges, true_separatrix, params);
else
	plotInference2D(inference_x, sample_x, [], sample_y, [], Alpha, Beta, ParameterRanges, true_separatrix, params);
end
drawnow

% Begin iteration
for iteration = 2:params.Number_Of_Iterations
	fprintf('Beginning iteration %d of %d with %d samples\n', iteration, params.Number_Of_Iterations, length(sample_y));
	
	% Use igBDOE to generate new samples:
	new_sample_x = igBDOE(sample_x, sample_y, inference_x, ParameterRanges, params);
	new_sample_y = model.Sample(new_sample_x);	% Evaluate model at new samples

	% Performn Separatrix inference for plotting
	[Alpha, Beta] = SeparatrixInference([sample_x; new_sample_x], [sample_y; new_sample_y], inference_x, params);
	
	% Plotting
	h = figure(3); clf; set(h, 'color', 'w', 'name', 'Separatrix Analysis');
	if params.Num_Dimensions == 1
		plotInference1D(inference_x, sample_x, new_sample_x, sample_y, new_sample_y, Alpha, Beta, ParameterRanges, true_separatrix, params);
	else
		plotInference2D(inference_x, sample_x, new_sample_x, sample_y, new_sample_y, Alpha, Beta, ParameterRanges, true_separatrix, params);
	end
	
	% Join new samples with old
	sample_x = [sample_x; new_sample_x];
	sample_y = [sample_y; new_sample_y];
end

disp('Done');

end