% SeparatrixInference performs the inferece step of the Separatrix Algorithm.
% Inputs:
%  sample_x: (Number of samples) by (number of dimensions) of sample points
%  sample_y: (Number of samples) by one of {0,1} results
%  inference_x: (Number of samples) by (number of dimensions) of inference points
%  params: parameters structure
%
% Outputs:
%  alpha: Effective number of successes at each inference point, respectively
%  beta: Effective number of failures at each inference point, respectively
%
% The resulting success probability distribution at each inference point
% can be computed by plugging alpha and beta into a Beta distribution.
%
% Separatrix Demo, Institute for Disease Modeling, May 2014

function [alpha, beta] = SeparatrixInference(sample_x, sample_y, inference_x, params)

%% Number of sample points, inference points, and dimensions
Nsamp = size(sample_x, 1);
Ninference = size(inference_x, 1);
D = params.Num_Dimensions;

%% Compute the density of sample points at each sample point

% Compute squared distance from each sample point to the k-th
% nearest neighboring sample point (eqn 5)
kNN_rho = max( params.Min_Neighbors_for_KNN, floor(params.c_rho * Nsamp^params.gamma_rho) );
D2_samp_samp = computeD2(sample_x, sample_x);	% Matrix of squared distances
D2_samp_samp_sorted = sort(D2_samp_samp);
D_kNN_rho = sqrt(D2_samp_samp_sorted( sub2ind([Nsamp Nsamp], (kNN_rho+1)*ones(1,Nsamp), 1:Nsamp) ))';

%  Compute density
rho_xx = zeros(Nsamp, 1);
for j = 1:Nsamp
	rho_xx(j) = (2*pi)^(-D/2) * mean( (params.h_rho*D_kNN_rho).^(-D) .* ...
		exp( -1/2 * D2_samp_samp(:,j)./(params.h_rho*D_kNN_rho).^2 ) );
end

%% Apply the density-corrected kernel

% Compute squared distance from each inference point to to the k-th
% nearest neighboring sample point (eqn 5)
D2_samp_inf = computeD2(sample_x, inference_x);
% Compute number of sample points at each inference point using a
% scale model (eqn 6)
Nhat_inf = sum( exp( -1/(2*params.Scale_Model_Sigma^2) * D2_samp_inf) )';
kNN_inference = max( params.Min_Neighbors_for_KNN, floor(params.c_inference * Nhat_inf.^params.Gamma_Inference) );
D2_samp_inf_sorted = sort(D2_samp_inf);
D_kNN_inf = sqrt(D2_samp_inf_sorted( sub2ind([Nsamp Ninference], (kNN_inference+1)', 1:Ninference) ))';
D_kNN_inf = min( params.max_g, max( params.min_g, D_kNN_inf));

% Compute the kernel and apply to get alpha and beta
kernel = diag(1./rho_xx) * exp( -1/2 * D2_samp_inf * diag(1./D_kNN_inf.^2) );
alpha = kernel'*sample_y;
beta = kernel'*(1-sample_y);

% Mu is a scaling factor so that alpha + beta = Nhat
mu = Nhat_inf ./ (alpha + beta);
alpha = mu .* alpha + 1;	% Add one for Beta distribution
beta = mu .* beta + 1;

end