% igBDOE performs the "interest guided Bayesian design of experiments"
% step of the Separatrix Algorithm.
%
% Inputs:
%  sample_x: (Number of samples) by (number of dimensions) of sample points
%  sample_y: (Number of samples) by one of {0,1} results
%  inference_x: (Number of samples) by (number of dimensions) of inference points
%  ParameterRanges: Struct of min and max of each parameter, min=0 and max=1 required for this implementation
%  params: structure of fixed parameters
%
% Outputs:
%  new_sample_x: Num_Next_Samples by (number of dimensions) matrix of new points to sample
%
% Separatrix Demo, Institute for Disease Modeling, May 2014

function new_sample_x = igBDOE(sample_x, sample_y, inference_x, ParameterRanges, params)

NumPoints = size(sample_x, 1);

% Read parameters from config:
NumNextPoints = params.Num_Next_Samples;

% Determine how many LHS points to take:
NumLHS = max(...
	params.Num_Next_Samples - min( floor(params.Max_Fraction_igBDOE*NumPoints), NumNextPoints), ...
	floor(params.Fraction_LHS * NumNextPoints) ...
	);

% Remaining points to be computed using BDOE:
NumNextPoints = NumNextPoints - NumLHS;

% If all LHS, take shortcut and return:
if NumNextPoints <= 0
	new_sample_x = LHSPointSelection(params.Num_Next_Samples, params.Num_Dimensions, ParameterRanges);
	return
end

% Load from previous iteration's MCMC (mcmcTestPoints.mat) or use LHS if first iteration
fn = fullfile(params.Output_Dir, 'mcmcTestPoints.mat');
testPoints = loadOrCreatePoints(fn, params.Num_Test_Points);

% Load from previous iteration's MCMC (mcmcPossibleSamplePoints.mat) or use LHS if first iteration
fn = fullfile(params.Output_Dir, 'mcmcPossibleSamplePoints.mat');
possibleSamplePoints = loadOrCreatePoints(fn, params.Num_Candidates_Points);

% We're going to compare the inference at the test points before and after
% addition of each possible sample point.  The outcome at each possible
% sample point will be first a success and then repeated with failure.  The
% expected interest-guided KL divergence is computed based on these
% results.

% Compute inference at possible sample points, used to compute mu (the
% expected success probability at each possible sample point)
[mu_alpha, mu_beta] = SeparatrixInference(sample_x, sample_y, possibleSamplePoints, params);

% This is just for the MCMC selection of test and possible sample points
% for the next iteration:
[alpha, beta] = SeparatrixInference(sample_x, sample_y, inference_x, params);
InterestDistributionInferencePoints = computeInterestDistribution(alpha, beta, params.Interest_Level);

% Now compute the (base) interest distribution at the test points:
[alpha_test, beta_test] = SeparatrixInference(sample_x, sample_y, testPoints, params);
InterestDistributionTestPoints = computeInterestDistribution(alpha_test, beta_test, params.Interest_Level);

% Compute expected KL divergence at each possible sample point
NpossibleSamplePoints = size(possibleSamplePoints,1);
ED_KL = zeros(NpossibleSamplePoints,1);

for i = 1:NpossibleSamplePoints
	
	% Recompute inference with success at ith possible sample point
	[alpha_success, beta_success] = SeparatrixInference([sample_x; possibleSamplePoints(i,:)], [sample_y; 1], testPoints, params);
	
	% Recompute the interest distribution with the success
	InterestDistributionSuccess = computeInterestDistribution(alpha_success, beta_success, params.Interest_Level);
	D_KL_success = computeKL(InterestDistributionTestPoints, InterestDistributionSuccess);
	
	% Recompute inference with failure at ith possible sample point
	[alpha_fail, beta_fail] = SeparatrixInference([sample_x; possibleSamplePoints(i,:)], [sample_y; 0], testPoints, params);
	
	% Recompute the interest distribution with the failure
	InterestDistributionFail = computeInterestDistribution(alpha_fail, beta_fail, params.Interest_Level);
	D_KL_failure = computeKL(InterestDistributionTestPoints, InterestDistributionFail);
	
	% Compute the mean and record the expectation of D_KL
	mu = mu_alpha(i) / (mu_alpha(i) + mu_beta(i));
	ED_KL(i) = mu * D_KL_success + (1-mu)*D_KL_failure;
end

% Choose next sample points based on expected KL divergence
[~,order] = sort(ED_KL, 'descend');
NextPointsInds = order(1:NumNextPoints);
new_sample_x = possibleSamplePoints(NextPointsInds,:);

% Add in LHS, if any
if NumLHS>0
	new_sample_x = [new_sample_x; LHSPointSelection(NumLHS, params.Num_Dimensions, ParameterRanges)];
end

% MCMC to distribute from posterior of interest variance
[mcmcTestPoints, mcmcPossibleSamplePoints] = ...
	ChooseTestAndSamplePointsForNextIteration(testPoints, inference_x, InterestDistributionTestPoints, InterestDistributionInferencePoints, possibleSamplePoints, ParameterRanges, params);

% Save for next iteration
save( fullfile( params.Output_Dir, 'mcmcTestPoints.mat'), 'mcmcTestPoints' );
save( fullfile( params.Output_Dir, 'mcmcPossibleSamplePoints.mat'), 'mcmcPossibleSamplePoints' );


%% HELPER FUNCTIONS:

	function points = loadOrCreatePoints( fn, Num_Points)
		% Load mat or use LHS to create points
		if exist( fn, 'file')
			tmp = load(fn);
			fields = fieldnames(tmp);
			points = tmp.(fields{1});
		else
			points = LHSPointSelection(Num_Points, params.Num_Dimensions, ParameterRanges);
		end
		
		% Hit the corners (required for current MCMC implementation)
		points = zeroCorners(points);
	end

	function pts = zeroCorners(pts)
		Ndim = size(pts, 2);
		vec = zeros(1,Ndim);
		cnt = 1;
		while sum(vec) < Ndim
			if ~any(ismember(pts, vec, 'rows'))
				pts(cnt,:) = vec;
				cnt = cnt + 1;
			end
			vec = increment(vec);
		end
		if ~any(ismember(pts, vec, 'rows'))
			pts(cnt,:) = vec;
		end
	end

	function vec = increment(vec)
		fz = length(vec);
		while vec(fz) == 1
			vec(fz) = 0;
			fz = fz - 1;
		end
		vec(fz) = 1;
	end
end
